package de.markusheiden.vaultconverter;

import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.io.Reader;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

/**
 * Vault converter for war3FT.
 * Converts vault.ini to sql.
 */
public class VaultConverter
{
  /**
   * Main.
   * Usage VaultConverter [vault.ini [vault.sql [tablename]]]
   *
   * @param args
   */
  public static void main (String[] args)
  {
    VaultConverter converter = new VaultConverter();
    try
    {
      String filename = "./vault.ini";
      String tablename = "war3users";
      PrintStream out = System.out;
      if (args.length > 0 && "--help".equals(args[0]))
      {
        System.out.println("Usage:");
        System.out.println("java -jar converter.jar [vault.ini [tablename [vault.sql]]]");
        System.out.println("  vault.ini    input file, defaults to vault.ini");
        System.out.println("  tablename    table name for sql inserts, defaults to war3users");
        System.out.println("  vault.sql    output file for generated sql, defaults to standard out");
      }
      else
      {
        if (args.length >= 1)
        {
          filename = args[0];
        }
        if (args.length >= 2)
        {
          tablename = args[1];
        }
        if (args.length >= 3)
        {
          out = new PrintStream(new FileOutputStream(args[2]));
        }

        converter.convert(new FileReader(filename), tablename, out);
      }
    }
    catch (IOException e)
    {
      e.printStackTrace(System.err);
    }
  }

  /**
   * Convert vault to sql.
   *
   * @param vault reader for vault.ini
   * @param tablename table name for sql insert
   * @param out output stream
   */
  public void convert (Reader vault, String tablename, PrintStream out) throws IOException
  {
    assert vault != null: "Precondition: vault != null";
    assert out != null: "Precondition: out != null";

    Calendar calendar = Calendar.getInstance();
    calendar.setTime(new Date());
    int year = calendar.get(Calendar.YEAR);

    int lines = 0;
    int converted = 0;
    LineNumberReader reader = new LineNumberReader(vault);

    String line;
    while ((line = reader.readLine()) != null)
    {
      lines++;

      line = line.trim();

      if (line.startsWith(";"))
      {
        System.err.println("Ignored comment line " + reader.getLineNumber());
      }
      else
      {
        String[] columns = columns(line);
        int length = columns.length;
        if (length < 14)
        {
          System.err.println("Ignored short line " + reader.getLineNumber() + ": " + line);
        }
        else
        {
          String key = columns[0];
          String id = columns[1];
          String xp = columns[2];
          String race = columns[3];
          String skill1 = columns[4];
          String skill2 = columns[5];
          String skill3 = columns[6];
          String skill4 = columns[7];

          String name = merge(columns, 8, length - 6);

//            String ip = columns[length - 5];
          String month = columns[length - 4];
          String day = columns[length - 3];
          String hour = columns[length - 2];
          String minute = columns[length - 1];

          if (!key.equals(id + "_" + race))
          {
            System.err.println("Ignored line with incorrect key " + reader.getLineNumber() + ": " + line);
          }
          else
          {
            StringBuffer result = new StringBuffer(256);
            result.append("INSERT INTO ");
            result.append(tablename);
            result.append(" (playerid, playername, xp, race, skill1, skill2, skill3, skill4, time) VALUES (");
            quote(result, id);
            result.append(", ");
            quote(result, name);
            result.append(", ");
            quote(result, xp);
            result.append(", ");
            quote(result, race);
            result.append(", ");
            quote(result, skill1);
            result.append(", ");
            quote(result, skill2);
            result.append(", ");
            quote(result, skill3);
            result.append(", ");
            quote(result, skill4);
            result.append(", ");
            quote(result, year + "-" + month + "-" + day + " " + hour + ":" + minute + ":00");
            result.append(");");

            out.println(result.toString());

            converted++;
          }
        }
      }
    }

    System.err.println("Converted " + converted + "/" + lines + " lines.");
  }

  /**
   * Split up line into columns.
   *
   * @param line line
   */
  public String[] columns (String line)
  {
    assert line != null: "Precondition: line != null";

    StringTokenizer tokenizer = new StringTokenizer(line, " \t");
    String[] result = new String[tokenizer.countTokens()];
    for (int i = 0; i < result.length; i++)
    {
      result[i] = tokenizer.nextToken().trim();
    }

    assert result != null: "Postcondition: result != null";
    return result;
  }

  /**
   * Merge some columns.
   *
   * @param columns columns
   * @param start start column to merge
   * @param end end column to merge
   */
  public String merge (String[] columns, int start, int end)
  {
    assert columns != null: "Precondition: columns != null";
    assert end >= start: "Precondition: end >= start";

    StringBuffer result = new StringBuffer(100);
    for (int i = start; i <= end; i++)
    {
      result.append(columns[i]);
      result.append(" ");
    }
    result.setLength(result.length() - 1);

    return result.toString();
  }

  /**
   * Quote a string.
   */
  public void quote (StringBuffer buffer, String string)
  {
    assert buffer != null: "Precondition: buffer != null";
    assert string != null: "Precondition: string != null";

    buffer.append("'");
    buffer.append(string.replace("\'", "\\'"));
    buffer.append("'");
  }
}
